#!/usr/bin/perl

use strict;
use warnings;

##
# Pre process a VDM++ class in order to use it on the Automatic Proof Support system.
# Consists on the following operations:
#       -- VDM++ toolbox proof obligation file generation
#       -- po_extractor parses the pog file, and generates a new VDM++ class with the
#          proof obligation's expressions as bool values
#       -- parse the new (values) class with overture parser
#       -- po_AST_extractor parses the AST, generates a VDM set with the portions 
#          relative to the values (OmlValueShape)
# 
# The last part of the pre-processing is done in The VDM++ Toolbox with the help
# of the vdm_addon distributed with this script.
##
my $filename = $ARGV[0] 
    or die "No input file";
chomp $filename;

# file for VDM++ class with proof obligation's expressions as bool values
#my $POVALUES="$filename.values.vpp";

# directory where to find the Overture Parser
my $OVT_PARSER="/Users/gentux/root/opt/overture/parser";
die "OvertureTool not found" unless -d $OVT_PARSER;

#directory where to find the VDM++ Toolbox
my $VDM_TOOLS="/Users/gentux/root/opt/vdmpp";
die "VDMTool not found" unless -d $VDM_TOOLS;

my $javalibs = "$OVT_PARSER/VDM.jar:$OVT_PARSER/org.overturetool.parser.jar";
my $overture = "org.overturetool.parser.imp.TestParser";

# generate the proof obligations
qx{$VDM_TOOLS/bin/vppde -t $filename}
   or die "Invalid VDM Model";
qx{$VDM_TOOLS/bin/vppde -g $filename}
   or die "VDM Error";

# generate the Oml AST of the model
my $OmlAST = qx{java -cp "$javalibs" $overture -pp $filename}
    or die "Overture Error";

#############################################################################
my $filehandler;
#my $tmpfilename="tmp.app";
my @contents=split "\n" => $OmlAST;

#open($filehandler, "< $tmpfilename") or die "Can't open \"$tmpfilename\"";
#while (<$filehandler>) {push @contents => $_;}
#close($filehandler);

shift @contents;
shift @contents;
shift @contents;
shift @contents;
pop @contents;

open ($filehandler, "> $filename.app")
    or die "Can't open \"$filename.app\"";
for (@contents) { print $filehandler => "$_\n";}
close($filehandler);

#system( "rm tmp.app tmp2.app lines");
########

# generate a vpp file with the proof obligations as bool type values
# java  ProofObligationExtractor $*
qx{java ProofObligationExtractor $filename.pog > $filename.values.vpp}
   or die "poExtractor Error";

# parse the generated poValues.vpp file
my $poValues = qx{java -cp "$javalibs" $overture -pp $filename.values.vpp}
    or die "Overture Error";

#####################################################################
@contents= split "\n" => $poValues;
#open($filehandler, "< $tmpfilename") or die "Can't open \"$tmpfilename\"";
#while (<$filehandler>) {push @contents => $_;}
#close($filehandler);

shift @contents;
shift @contents;
shift @contents;
shift @contents;
pop @contents;

open ($filehandler, "> $filename.values.vpp.app")
    or die "Can't open \"$filename.values.vpp.app\"";
for (@contents) {print $filehandler => "$_\n";}
close($filehandler);

# generate valushape set and proof obligation's type map
# java  ProofObligationASTExtractor $*  
qx{java ProofObligationASTExtractor $filename.values.vpp.app > $filename.vset}
   or die "poASTExtractor Error";

#clean temporary files
#system "rm *.pog *.values.vpp* lines";


