import java.util.*;
import java.lang.*;
import java.io.*;

/**
 * <p>Implementation of the ProofObligationASTExtractor.</p>
 * <p>Read input Proof Obligation file generated by The VDM++ Toolbox,
 * and generate a VDM++ class with each Proof Obligation's expression
 * as a bool type value. Also generate a VDM mapping from Proof Obligation
 * identifier to its type.</p>
 *
 * @author Miguel Ferreira <b>miguel@di.uminho.pt</b>
 * @author Samuel Silva <b>silva.samuel@gmail.com</b>
 * @version 1.0
 */
public class ProofObligationASTExtractorLogic
{
	// list to store the diferent Proof Obligations
	private LinkedList<StringBuffer> po_list 
		= new LinkedList<StringBuffer>();

	// store the lines of each Proof Obligation
	private StringBuffer po = new StringBuffer();

	// count the number os '(' that need a matching ')' 
	private int count = 0;
	
	// true if reached the end of the OmlAccessDefinition object
	boolean endOmlAccessDefinition = false;

	/**
	 * Constructor.
	 */
	ProofObligationASTExtractorLogic () { }
	
	/**
	 * Process a line obtained in the YYINITIAL state.
	 *
	 * @param line the line to process.
	 * @return true if reached the end of a OmlAccessDefinition or a
	 *              OmlValueShape.
	 */
	public boolean invalueLINE(String line) {
		
		boolean result = false;
		
		if(endOmlAccessDefinition) {
			// collect the entire line
			po.append(line);
		}

		// count '('
		String[] parts = line.split("[(]");
		int found = parts.length - 1;
		if(found > 0) {
			// line has some '('
			incrementOpenCount(found);
		}

		// count ')'
		parts = line.split("[)]");
		found = parts.length - 1;
		if(found > 0) {
			// line has some ')'
			decrementOpenCount(found);
		}
		
		// check if reached the end of the OmlAccessDefinition
		if((count == 1) && !endOmlAccessDefinition) {
			endOmlAccessDefinition = true;
		}
		// check if reched the end of the OmlValueShape
		else if((count == 1) && endOmlAccessDefinition) {

			// collect the value
			po_list.addLast(po);

			// clear the buffer for the next value
			po = new StringBuffer();

			// clear control variables
			endOmlAccessDefinition = false;
			count = 0;
			
			// reached the end of the Value delcaration
			result = true;
		}
		
		return result;
	}
	
	/**
	 * Process colected data in order to generate a VDM++ class with the 
	 * Proof Obligations as bool values.
	 */
	public void eof() {
		// generate a set with all OmlValueShape
		StringBuffer omlValueShapeSet = VppCodeGenerator.generateSet(po_list);
		
		// print output
		System.out.println(omlValueShapeSet.toString());
	}
	
	/**
	 * Get the Proof Obligation list.
	 *
	 * @return a LinkedList of Proof Obligation strings.
	 */
	public LinkedList<StringBuffer> getPOList() {
		return po_list;
	}
	
	/**
	 * Increment the '(' counter.
	 *
	 * @param value the value of the increment.
	 */
	public void incrementOpenCount(int value) {
		count += value;
	}
	
	/**
	 * Decrement the '(' counter.
	 *
	 * @param value the value of the decrement.
	 */
	public void decrementOpenCount(int value) {
		count -= value;
	}
	
	/**
	 * Increment the '(' counter by one.
	 */
	public void incrementOpenCount() {
		incrementOpenCount(1);
	}
	
	/**
	 * Decrement the '(' counter by one.
	 */
	public void decrementOpenCount() {
		decrementOpenCount(1);
	}
}
