import java.util.*;
import java.lang.*;
import java.io.*;

%%

%public
%class ProofObligationASTExtractor
%line
%char
%full
%unicode

%{

private ProofObligationASTExtractorLogic impl 
	= new ProofObligationASTExtractorLogic();



/**
   * Runs the scanner on input files.
   *
   * This is a standalone scanner, it will print any unmatched
   * text to System.out unchanged.
   *
   * @param argv   the command line, contains the filenames to run
   *               the scanner on.
   */
  public static void main(String argv[]) {
    if (argv.length == 0) {
      System.out.println("Usage : java ProofObligationExtractor <inputfile>");
    }
    else {
	  int i = 0;
	  String typesMapFile = null;
	
	  // check if should process types map file
	  if(argv.length > 3) {
		if(argv[1].equals("-tm")) {
			typesMapFile = argv[2];
			i = 3;
		}
	  }
      for (; i < argv.length; i++) {
        ProofObligationASTExtractor scanner = null;
        try {
		  if(typesMapFile != null) {
			scanner = new ProofObligationASTExtractor(
				new FileReader(argv[i]), typesMapFile);
		  }
		  else {
			scanner = new ProofObligationASTExtractor(
				new FileReader(argv[i]));
		  }
          
          while ( !scanner.zzAtEOF ) scanner.yylex();
        }
        catch (java.io.FileNotFoundException e) {
          System.out.println("File not found : \""+argv[i]+"\"");
        }
        catch (java.io.IOException e) {
          System.out.println("IO error scanning file \""+argv[i]+"\"");
          System.out.println(e);
        }
        catch (Exception e) {
          System.out.println("Unexpected exception:");
          e.printStackTrace();
        }
      }
    }
  }

/**
   * Creates a new scanner
   * There is also a java.io.InputStream version of this constructor.
   *
   * @param   in  the java.io.Reader to read input from.
   * @param   typesMapFile Proof Obligations types's mapping file
   */
  public ProofObligationASTExtractor(java.io.Reader in,
		String typesMapFile) throws java.io.FileNotFoundException,
		       java.io.IOException,
		 	   java.lang.ClassNotFoundException {
    this.zzReader = in;
	VppCodeGenerator.readTypesMap(typesMapFile);
  }

  /**
   * Creates a new scanner.
   * There is also java.io.Reader version of this constructor.
   *
   * @param   in  the java.io.Inputstream to read input from.
   * @param   typesMapFile Proof Obligations types's mapping file
   */
  public ProofObligationASTExtractor(java.io.InputStream in,
		String typesMapFile) throws java.io.FileNotFoundException,
		       java.io.IOException,
		 	   java.lang.ClassNotFoundException {
    this(new java.io.InputStreamReader(in), typesMapFile);
  }

%}

%state INVALUE
/*
 * INVALUE state is reached when the next lines will be part of a 
 * value declaration
 */

VALUE = [\t ]*"new OmlValueDefinition("[\n\r]+
LINE  = [^\n]*[\n\r]+

%%

<YYINITIAL>{VALUE}	{

	// got a value declaration
	// count the '(' found on the line
	impl.incrementOpenCount();

	// enter the INVALUE state
	yybegin(INVALUE);
}

<YYINITIAL>{LINE}	{

	// got a line
	// nothing to do so far
}

<INVALUE>{LINE}	{
	
	// got a Value line
	String line = yytext();
	
	boolean end = impl.invalueLINE(line);
	
	if(end) {
		// reached the end of the Value delcaration
		yybegin(YYINITIAL);
	}
}
	

<<EOF>> {

	// post processing code

	impl.eof();

	return null;
}
